﻿using ID3_Sample_Code_Test.ID3global;
using System;
using System.ServiceModel;

namespace ID3_Sample_Code_Test
{
    public static class ID3globalServiceCalls
    {
        public static GlobalAccount CheckCredentials(String binding, String username, String password)
        {
            using (GlobalCredentialsClient globalCredentialsClient = new GlobalCredentialsClient(binding))
            {
                try
                {
                    GlobalAccount checkCredentialsResult = globalCredentialsClient.CheckCredentials(username, password);
                    
                    globalCredentialsClient.Close();

                    return checkCredentialsResult;
                }
                catch (CommunicationException)
                {
                    globalCredentialsClient.Abort();
                }
                catch (TimeoutException)
                {
                    globalCredentialsClient.Abort();
                }
                catch (Exception)
                {
                    globalCredentialsClient.Abort();
                    throw;
                }

                return null;
            }
        }

        public static GlobalResultData AuthenticateSP(GlobalInputData globalInputData, String bindingName, GlobalProfileIDVersion profileGuid, String username, String password, String CustomerRef)
        {
            using (GlobalAuthenticateClient globalAuthenticateClient = new GlobalAuthenticateClient(bindingName))
            {
                try
                {
                    // Give our client the credentials we want to use to sign-on
                    globalAuthenticateClient.ClientCredentials.UserName.UserName = username;
                    globalAuthenticateClient.ClientCredentials.UserName.Password = password;

                    GlobalResultData results = globalAuthenticateClient.AuthenticateSP(profileGuid, CustomerRef, globalInputData);

                    globalAuthenticateClient.Close();

                    return results;

                    
                }
                catch (CommunicationException c)
                {
                    Console.WriteLine(c.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (TimeoutException t)
                {
                    Console.WriteLine(t.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                    globalAuthenticateClient.Abort();
                    throw;
                }

                return null;
            }
        }

        public static GlobalPEPIntelligenceData GetPEPIntelligence(String bindingName, String username, String password, Guid OrgID, String sanctionsID)
        {
            using (GlobalSearchClient globalAuthenticateClient = new GlobalSearchClient(bindingName))
            {
                try
                {
                    // Give our client the credentials we want to use to sign-on
                    globalAuthenticateClient.ClientCredentials.UserName.UserName = username;
                    globalAuthenticateClient.ClientCredentials.UserName.Password = password;

                    GlobalPEPIntelligenceData results = globalAuthenticateClient.GetPEPIntelligenceData(OrgID, sanctionsID);

                    globalAuthenticateClient.Close();

                    return results;
                }
                catch (CommunicationException c)
                {
                    Console.WriteLine(c.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (TimeoutException t)
                {
                    Console.WriteLine(t.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                    globalAuthenticateClient.Abort();
                    throw;
                }

                return null;
            }
        }

        public static GlobalSanctionsEnforcementData GetSanctionsIntelligence(String bindingName, String username, String password, Guid OrgID, String sanctionsID)
        {
            using (GlobalSearchClient globalAuthenticateClient = new GlobalSearchClient(bindingName))
            {
                try
                {
                    // Give our client the credentials we want to use to sign-on
                    globalAuthenticateClient.ClientCredentials.UserName.UserName = username;
                    globalAuthenticateClient.ClientCredentials.UserName.Password = password;

                    GlobalSanctionsEnforcementData results = globalAuthenticateClient.GetSanctionsEnforcementsData(OrgID, sanctionsID);

                    globalAuthenticateClient.Close();

                    return results;
                }
                catch (CommunicationException c)
                {
                    Console.WriteLine(c.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (TimeoutException t)
                {
                    Console.WriteLine(t.ToString());
                    globalAuthenticateClient.Abort();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                    globalAuthenticateClient.Abort();
                    throw;
                }

                return null;
            }
        }

        public static GlobalSanctionsIssuer[] GetSanctionsIssuer(String bindingName, String username, String password)
        {
            using (GlobalConfigClient globalConfigClient = new GlobalConfigClient(bindingName))
            {
                try
                {
                    // Give our client the credentials we want to use to sign-on
                    globalConfigClient.ClientCredentials.UserName.UserName = username;
                    globalConfigClient.ClientCredentials.UserName.Password = password;

                    GlobalSanctionsIssuer[] results = globalConfigClient.GetSanctionsIssuers();

                    globalConfigClient.Close();

                    return results;
                }
                catch (CommunicationException c)
                {
                    Console.WriteLine(c.ToString());
                    globalConfigClient.Abort();
                }
                catch (TimeoutException t)
                {
                    Console.WriteLine(t.ToString());
                    globalConfigClient.Abort();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                    globalConfigClient.Abort();
                    throw;
                }

                return null;
            }
        }
    }
}
