﻿using ID3_Sample_Code_Test.ID3global;
using System;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Xml;
using System.Xml.Linq;

namespace ID3_Sample_Code_Test
{
    public class XMLDeserialiser
    {
        public XMLDeserialiser()
        { 
        }

        public static GlobalInputData ConvertXmlRequestToGlobalInputData(string filepath)
        {
            //Extracts the input data from sample XML request for AuthenticateSP
            //to get this to work, you'll need to update the data contract attribute for 'GlobalInputData'; right click and 'Go to Definition', then update the attribute as per below:
            //[System.Runtime.Serialization.DataContractAttribute(Name="InputData", Namespace="http://www.id3global.com/ID3gWS/2013/04")]
            GlobalInputData inputData = null;

            try
            {
                XmlDocument soapenvelopexml = new XmlDocument();
                //soapenvelopexml.Load(filepath);

                using (StreamReader oReader = new StreamReader(filepath, Encoding.GetEncoding("ISO-8859-1")))
                {
                    soapenvelopexml.Load(oReader);
                }

                string xml = soapenvelopexml.InnerXml;

                const string topNamespace = "http://www.id3global.com/ID3gWS/2013/04", inputDataNode = "InputData";
                const StringComparison comparison = StringComparison.InvariantCultureIgnoreCase;
                var semiParsedData = XDocument.Parse(xml);


                XElement authenticateSPElement = semiParsedData.Descendants().Where(a => a.Name.Namespace.NamespaceName == topNamespace).FirstOrDefault();

                var secondLevelEntity = authenticateSPElement?.Descendants().FirstOrDefault(element =>
                    string.Equals(element.Name.Namespace.NamespaceName, topNamespace, comparison)
                    && string.Equals(element.Name.LocalName, inputDataNode, comparison));
                
                using (var resultReader = secondLevelEntity.CreateReader())
                {
                    var deserializer = new DataContractSerializer(typeof(GlobalInputData));
                    inputData = (GlobalInputData)deserializer.ReadObject(resultReader);

                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            return inputData;
        }

        public static Tuple<string,string> ExtractUsernameAndPasswordFromXML(string filepath)
        {
            //Extracts the username and password elements from ID3global AuthenticateSP request in XML format
            Tuple<string, string> creds = null;

            try
            {
                XmlDocument soapenvelopexml = new XmlDocument();
                //soapenvelopexml.Load(filepath);

                using (StreamReader oReader = new StreamReader(filepath, Encoding.GetEncoding("ISO-8859-1")))
                {
                    soapenvelopexml.Load(oReader);
                }

                string xml = soapenvelopexml.InnerXml;

                const string username = "Username", password = "Password";
                const StringComparison comparison = StringComparison.InvariantCultureIgnoreCase;
                var semiParsedData = XDocument.Parse(xml);

                var usernameEntity = semiParsedData.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.LocalName, username, comparison));

                var passwordEntity = semiParsedData.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.LocalName, password, comparison));
                
                creds = new Tuple<string, string>(usernameEntity.Value, passwordEntity.Value);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            return creds;
        }

        public static GlobalProfileIDVersion ExtractProfileGuidFromXML(string filepath)
        {
            //Extracts the profile elements from ID3global AuthenticateSP request in XML format
            GlobalProfileIDVersion globalProfileIDVersion = null;

            try
            {
                XmlDocument soapenvelopexml = new XmlDocument();
                //soapenvelopexml.Load(filepath);

                using (StreamReader oReader = new StreamReader(filepath, Encoding.GetEncoding("ISO-8859-1")))
                {
                    soapenvelopexml.Load(oReader);
                }

                string xml = soapenvelopexml.InnerXml;

                const string profileDetails = "ProfileIDVersion", id = "ID", version = "Version";
                const StringComparison comparison = StringComparison.InvariantCultureIgnoreCase;
                var semiParsedData = XDocument.Parse(xml);

                var profileIDEntity = semiParsedData.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.LocalName, profileDetails, comparison));

                var idEntity = profileIDEntity?.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.LocalName, id, comparison));

                var versionEntity = profileIDEntity?.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.LocalName, version, comparison));

                globalProfileIDVersion = new GlobalProfileIDVersion()
                {
                    Version = UInt16.Parse(versionEntity.Value),
                    ID = new Guid(idEntity.Value)
                };
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            return globalProfileIDVersion;
        }
        
        public static GlobalResultData ConvertXmlResponseToGlobalResultData(string filepath)
        {
            //To get this working, you will need to right-click on 'GlobalResultData', click 'Go to Definition', then alter the DataContractAttribute as follows:
            // [System.Runtime.Serialization.DataContractAttribute(Name= "AuthenticateSPResult", Namespace="http://www.id3global.com/ID3gWS/2013/04")]

            try
            {
                XmlDocument soapenvelopexml = new XmlDocument();

                //soapenvelopexml.Load(filepath); //-> doesn't work for European characters etc


                using (StreamReader oReader = new StreamReader(filepath, Encoding.GetEncoding("ISO-8859-1")))
                {
                    soapenvelopexml.Load(oReader);
                }

                string xml = soapenvelopexml.InnerXml;

                const string topNamespace = "http://www.id3global.com/ID3gWS/2013/04", firstNode = "AuthenticateSPResponse", secondNode = "AuthenticateSPResult";
                const StringComparison comparison = StringComparison.InvariantCultureIgnoreCase;

                var spd = System.IO.File.ReadAllText(filepath);

                var semiParsedData = XDocument.Parse(xml);

                var desc = semiParsedData.Descendants().ToList();

                var topLevelEntity = semiParsedData.Descendants().FirstOrDefault(element =>
                        string.Equals(element.Name.Namespace.NamespaceName, topNamespace, comparison)
                        && string.Equals(element.Name.LocalName, firstNode, comparison));

                var secondLevelEntity = topLevelEntity?.Descendants().FirstOrDefault(element =>
                    string.Equals(element.Name.Namespace.NamespaceName, topNamespace, comparison)
                    && string.Equals(element.Name.LocalName, secondNode, comparison));

                if (secondLevelEntity == null)
                    return null;

                using (var resultReader = secondLevelEntity.CreateReader())
                {
                    var deserializer = new DataContractSerializer(typeof(GlobalResultData));
                    var myData = (GlobalResultData)deserializer.ReadObject(resultReader);

                    return myData;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            return null;
        }
    }
}
